<?php
// Телеметрия системных ресурсов
// Orange Pi PC (Plus)
// lostar-igor (ASt)
//
// ver. 1.0	05.12.2016

// Параметры, передаваемые в скрипт
// option - наименование параметра, по которому необходимо получить данные
//			=general - общие данные о системе для шапки,
//			=cpu - загрузка процессора в %
//			=core0 - загрузка 0 ядра в %
//			=core1 - загрузка 1 ядра в %
//			=core2 - загрузка 2 ядра в %
//			=core3 - загрузка 3 ядра в %
//			=cpufreq - частота работы процессора в MHz
//			=temp - температура в град.C
//			=temp1 - температура в град.C
//			=mem - занято памяти в MB
//			=memperc - занято памяти в %
//			=rx - принятые данные по сети в kB
//			=tx - отправленные данные по сети в kB
//			=rxtx - сумма принятых и отправленные данные по сети в kB
// type -	=all - получить все значения параметра, 
//        	=last - получить последние значения параметра
// time - 	=sec - получить секундные значения
// 		  	=min - получить минутные значения
// 		  	=5min - получить 5-ти минутные значения
// 		  	=15min - получить 15-ти минутные значения

// Константы
// Файл с данными (секундные данные)
define("__InputFileSec__", "/var/www/telemetry/sec");
// Файл с данными (минутные данные)
define("__InputFileMin__", "/var/www/telemetry/min");
// Файл с данными (5-ти минутные данные)
define("__InputFile5Min__", "/var/www/telemetry/5min");
// Файл с данными (15-ти минутные данные)
define("__InputFile15Min__", "/var/www/telemetry/15min");
// Константы - номер позиции в строке с данными
// Дата в сек.
define("__PosDate__", 0);
// Загрузка процессора в % * 10
define("__PosCPU__", 1);
// Загрузка 0 ядра в % * 10
define("__PosCore0__", 2);
// Загрузка 1 ядра в % * 10
define("__PosCore1__", 3);
// Загрузка 2 ядра в % * 10
define("__PosCore2__", 4);
// Загрузка 3 ядра в % * 10
define("__PosCore3__", 5);
// Частота работы процессора в kHz
define("__PosCPUFreq__", 6);
// Температура в град.C
define("__PosTemp__", 7);
// Температура в град.C
define("__PosTemp1__", 8);
// Занято памяти в байтах
define("__PosMem__", 9);
// Использовано swap памяти в kB
define("__PosSwap__", 10);
// Использовано hdd в kB
define("__PosHdd__", 11);
// Принятые данные по сети в байтах
define("__PosRX__", 12);
// Отправленные данные по сети в байтах
define("__PosTX__", 13);
// Время работы системы в секундах
define("__PosUptime__", 14);

function result() {
	global $Data, $memTotal;
	switch ($_GET[option]) {
		case "cpu":
			// Загрузка процессора в XX.X%
			$a = $Data[__PosCPU__] / 10;
			return $a;
		break;
		case "core0":
			// Загрузка ядра 0 в XX.X%
			$a = $Data[__PosCore0__] / 10;
			return $a;
		break;
		case "core1":
			// Загрузка ядра 1 в XX.X%
			$a = $Data[__PosCore1__] / 10;
			return $a;
		break;
		case "core2":
			// Загрузка ядра 2 в XX.X%
			$a = $Data[__PosCore2__] / 10;
			return $a;
		break;
		case "core3":
			// Загрузка ядра 3 в XX.X%
			$a = $Data[__PosCore3__] / 10;
			return $a;
		break;
		case "cpufreq":
			// Частота процессора в MHz
			$a = $Data[__PosCPUFreq__] / 1000;
			return $a;
		break;
		case "temp":
			// Температура в град.C
			$a = $Data[__PosTemp__] + 0;
			return $a;
		break;
		case "temp1":
			// Температура в град.C
			$a = $Data[__PosTemp1__] + 0;
			return $a;
		break;
		case "mem":
			// Занято памяти в MB
			$a = round($Data[__PosMem__] / 1024 / 1024, 2);
			return $a;
		break;
		case "memperc":
			// Занято памяти в XX.X%
			$a = round($Data[__PosMem__] / $memTotal * 100, 1);
			return $a;
		break;
		case "rx":
			// Принятые данные в kB
			$a = round($Data[__PosRX__] / 1024, 1);
			return $a;
		case "tx":
			// Переданные данные в kB
			$a = round($Data[__PosTX__] / 1024, 1);
			return $a;
		break;
		case "rxtx":
			// Сумма принятых и переданных данные в kB
			$a = round(($Data[__PosRX__] + $Data[__PosTX__]) / 1024, 1);
			return $a;
		break;
	}
}

// Читаем данные о системе для шапки
if ($_GET[option] == 'general') {
	// Массив, в который собираем необходимые данные
	$globalarray = array();
	if ($_GET[type] == 'all') {
		// hostname
		$hostname = exec("cat /etc/hostname");
		$globalarray[] = array("hostname" => $hostname);
		// MAC адрес
		$mac = exec("ifconfig | grep HWaddr | awk '{print $5}'");
		$globalarray[] = array("mac" => $mac);
		// IP адрес
		$ip = exec("ifconfig | grep Bcast | awk '{print $2}'");
		$ip = substr($ip, 5);
		$globalarray[] = array("ip" => $ip);
		// Название системы
		$nameos = exec("cat /etc/os-release | grep -w NAME");
		$nameos = substr($nameos, 5);
		$nameos = str_replace('"', '', $nameos);
		$globalarray[] = array("nameos" => $nameos);
		// Версия системы
		$versionos = exec("cat /etc/os-release | grep -w VERSION");
		$versionos = substr($versionos, 8);
		$versionos = str_replace('"', '', $versionos);
		$globalarray[] = array("versionos" => $versionos);
		// Kernel
		$kernel = exec("uname -rm");
		$globalarray[] = array("kernel" => $kernel);
		// Объем памяти в MB
		$ramtotal = exec("free -m | grep Mem | awk '{print $2}'");
		$globalarray[] = array("ramtotal" => $ramtotal);
		// Объем HDD отведенного под систему в kB (правильно указать имя устройства) 
		$hddtotal = exec("df | grep sdb2 | awk '{print $2}'");
		// Переводим в GB и округляем до сотых
		$hddtotal = round($hddtotal / 1024 / 1024, 2);
		$globalarray[] = array("hddtotal" => $hddtotal);
		// Объем swap в MB
		$swaptotal = exec("free -m | grep Swap | awk '{print $2}'");
		$globalarray[] = array("swaptotal" => $swaptotal);
	} else if ($_GET[type] == 'last') {
		// Читаем файл с параметрами
		$DataFile = file(__InputFileSec__);
		// Если в файле есть строки, то разбираем
		if (count($DataFile) > 0) {
			// Забираем последнюю строку считанного файла в массив
			$Data = explode (chr(32), $DataFile[count($DataFile)-1]);
			// Исползованная память в МБ
			$ramused = round($Data[__PosMem__] / 1024 / 1024, 2);
			// Объем памяти в MB
			$ramtotal = exec("free -m | grep Mem | awk '{print $2}'");
			$globalarray[] = array("ramused" => $ramused);
			$globalarray[] = array("ramusedperc" => round($ramused / $ramtotal *100, 2).'%');
			// Исползованное дисковое пространство в ГБ
			$hddused = round($Data[__PosHdd__] / 1024 / 1024, 2);
			// Объем HDD отведенного под систему в kB (правильно указать имя устройства) 
			$hddtotal = exec("df | grep sdb2 | awk '{print $2}'");
			// Переводим в GB и округляем до сотых
			$hddtotal = round($hddtotal / 1024 / 1024, 2);
			$globalarray[] = array("hddused" => $hddused);
			$globalarray[] = array("hddusedperc" => round($hddused / $hddtotal *100, 2).'%');
			// Исползованная swap память в МБ
			$swapused = round($Data[__PosSwap__] / 1024 / 1024, 2);
			// Объем swap в MB
			$swaptotal = exec("free -m | grep Swap | awk '{print $2}'");
			$globalarray[] = array("swapused" => $swapused);
			$globalarray[] = array("swapusedperc" => round($swapused / $swaptotal *100, 2).'%');
			// Uptime системы
			$uptime = $Data[__PosUptime__];
			// считаем  годы
			$year = floor($uptime / 31556926);
			$yearsec = $year * 31556926;
			// считаем месяцы
			$month = floor(($uptime - $yearsec) / 2629743);
			$monthsec = $month * 2629743;
			// считаем недели
			$week = floor(($uptime - $yearsec - $monthsec) / 604800);
			$weeksec = $week * 604800;
			// считаем дни
			$day = floor(($uptime - $yearsec - $monthsec - $weeksec) / 86400);
			$daysec = $day * 86400;
			// считаем часы
			$hour = floor(($uptime - $yearsec - $monthsec - $weeksec - $daysec) / 3600);
			$hoursec = $hour * 3600;
			// считаем минуты
			$minute = floor(($uptime - $yearsec - $monthsec - $weeksec - $daysec - $hoursec) / 60);
			$minutesec = $minute * 60;
			// считаем секунды
			$second = floor($uptime - $yearsec - $monthsec - $weeksec - $daysec - $hoursec - $minutesec);

			//$year = 1; $month = 12; $week = 2;
			// год, года, лет
			$yearstr = 'лет';
			if ($year == 1) {
				$yearstr = ' год';
			} else if (($year > 1) && ($year < 5)) {
				$yearstr = ' года';
			} else {
				$yearstr = ' лет';
			}
			// месяц, месяца, месяцев
			$monthstr = 'месяцев';
			if ($month == 1) {
				$monthstr = ' месяц';
			} else if (($month > 1) && ($month < 5)) {
				$monthstr = ' месяца';
			} else {
				$monthstr = ' месяцев';
			}
			// неделя, недели, недель
			$weekstr = 'недель';
			if ($week == 1) {
				$weekstr = ' неделя';
			} else if (($week > 1) && ($week < 5)) {
				$weekstr = ' недели';
			} else {
				$weekstr = ' недель';
			}
			// день, дня, дней
			$daystr = 'дней';
			if ($day == 1) {
				$daystr = ' день';
			} else if (($day > 1) && ($day < 5)) {
				$daystr = ' дня';
			} else {
				$daystr = ' дней';
			}
			$uptime = 'Uptime:<br />';
			// Передаем на страницу значения > 0
			if ($year > 0) {
				$uptime = $uptime.$year.$yearstr.', ';
			}
			if ($month > 0) {
				$uptime = $uptime.$month.$monthstr.', ';
			}
			if ($week > 0) {
				$uptime = $uptime.$week.$weekstr.',<br />';
			}
			if ($day > 0) {
				$uptime = $uptime.$day.$daystr.', ';
			}
			$globalarray[] = array("uptime" => $uptime.sprintf("%02d:%02d:%02d", $hour, $minute, $second));
		}
	}
	// Передаем подготовленные данные
	echo json_encode($globalarray);
} else {
	// Читаем файл с необходимым временным интервалом в массив
	switch ($_GET[time]) {
		case "sec":
			$DataFile = file(__InputFileSec__);
		break;
		case "min":
			$DataFile = file(__InputFileMin__);
		break;
		case "5min":
			$DataFile = file(__InputFile5Min__);
		break;
		case "15min":
			$DataFile = file(__InputFile15Min__);
		break;
	}

	// Если в файле есть строки, то разбираем
	if (count($DataFile) > 0) {
		if ($_GET[option] == 'memperc') {
			// Читаем общий объем памяти в байтах
			$memTotal = exec("free -b | grep Mem | awk '{print $2}'");
		}
		// Массив, в который собираем необходимые данные
		$dataarray = array();
		// Данные выбираем в зависимости от параметра type
		switch ($_GET[type]) {
			case "all":
				foreach ($DataFile as $key => $value) {
					// Забираем строку в массив
					$Data = explode (chr(32), $value);
					// Формируем выходные данные в зависимости от параметра 'option'
					// Время измерения параметра в ms
					$temp[0] = $Data[__PosDate__] * 1000;
					// Значение параметра
					$temp[1] = result();
					$dataarray[] = $temp;
				}
				// Передаем подготовленные данные
				echo json_encode($dataarray);
			break;
			case "last":
				// Забираем последнюю строку в массив
				$Data = explode (chr(32), $DataFile[count($DataFile)-1]);
				// Формируем выходные данные в зависимости от параметра 'option' 
				echo("[".$Data[__PosDate__] * 1000);
				echo(",".result()."]");
			break;
		}
	}
}
?>