#!/bin/bash

######################################################
# Телеметрия системных ресурсов                      #
# Orange Pi PC (Plus)                                #
#                                                    #
# lostar-igor (ASt)                                  #
#                                                    #
# ver 1.00  27.11.2016                               #
#                                                    #
#   0 - время                                        #
#   1 - загрузка процессора                          #
#   2 - загрузка ядра 0                              #
#   3 - загрузка ядра 1                              #
#   4 - загрузка ядра 2                              #
#   5 - загрузка ядра 3                              #
#   6 - частота процессора                           #
#   7 - температура в зоне 0                         #
#   8 - температура в зоне 1                         #
#   9 - использованная память в байтах               #
#  10 - использованная swap память в килобайтах      #
#  11 - использованное пространство hdd в килобайтах #
#  12 - количество принятых байтов по сети           #
#  13 - количество переданых байтов по сети          #
#  14 - время работы системы                         #
#                                                    #
######################################################

# Время работы системы в секундах
# /proc/uptime
# Данные о памяти (в байтах)
# free -b
# Данные об использованном пространстве HDD
# df
# Данные о приятых байтах
rx=/sys/devices/platform/gmac0/net/eth0/statistics/rx_bytes
# Данные об отправленных байтах
tx=/sys/devices/platform/gmac0/net/eth0/statistics/tx_bytes
# Данные о температуре
tempzone0=/sys/devices/virtual/thermal/thermal_zone0/temp
tempzone1=/sys/devices/virtual/thermal/thermal_zone1/temp
# Данные о текущей частоте процессора
cpufreqcur=/sys/devices/system/cpu/cpu0/cpufreq/cpuinfo_cur_freq
#scalingfreqcur=/sys/devices/system/cpu/cpu0/cpufreq/scaling_cur_freq
# Данные о загрузке процессора и его ядер
loadcpu=/proc/stat
# Секундные измерения
outsec=/var/www/telemetry/sec
# Минутные измерения
outmin=/var/www/telemetry/min
# 5-ти минутные измерения
out5min=/var/www/telemetry/5min
# 15-ти минутные измерения
out15min=/var/www/telemetry/15min
# Временный файл
temp=/var/www/telemetry/temp

# Количество хранения секундных измерений
hs=120
# Количество хранения минутных измерений
hm=120
# Количество хранения 5-ти минутных измерений
h5m=144
# Количество хранения 15-ти минутных измерений
h15m=192

# Время для минутных отсчетов
tm=$(date +%s)

# Время для 5-ти минутных отсчетов
t5m=$tm

# Время для 15-ти минутных отсчетов
t15m=$tm

# Читаем текущие значения принятых / переданных байт для расчета
# Секундные данные
rxs_prev=$(cat $rx)
txs_prev=$(cat $tx)
# Минутные данные
rxm_prev=$rxs_prev
txm_prev=$txs_prev
# 5-ти минутные данные
rx5m_prev=$rxs_prev
tx5m_prev=$txs_prev
# 15-ти минутные данные
rx15m_prev=$rxs_prev
tx15m_prev=$txs_prev

# Читаем текущее значение загрузки процессора
cpu=$(cat $loadcpu | grep cpu)

# Секундные данные
cpubs_prev=$(echo $cpu | awk '{print ($2+$3+$4)}')
cpuws_prev=$(echo $cpu | awk '{print ($2+$3+$4+$5)}')
cpu0bs_prev=$(echo $cpu | awk '{print ($13+$14+$15)}')
cpu0ws_prev=$(echo $cpu | awk '{print ($13+$14+$15+$16)}')
cpu1bs_prev=$(echo $cpu | awk '{print ($24+$25+$26)}')
cpu1ws_prev=$(echo $cpu | awk '{print ($24+$25+$26+$27)}')
cpu2bs_prev=$(echo $cpu | awk '{print ($35+$36+$37)}')
cpu2ws_prev=$(echo $cpu | awk '{print ($35+$36+$37+$38)}')
cpu3bs_prev=$(echo $cpu | awk '{print ($46+$47+$48)}')
cpu3ws_prev=$(echo $cpu | awk '{print ($46+$47+$48+$49)}')

# Минутные данные
cpubm_prev=$cpubs_prev
cpuwm_prev=$cpuws_prev
cpu0bm_prev=$cpu0bs_prev
cpu0wm_prev=$cpu0ws_prev
cpu1bm_prev=$cpu1bs_prev
cpu1wm_prev=$cpu1ws_prev
cpu2bm_prev=$cpu2bs_prev
cpu2wm_prev=$cpu2ws_prev
cpu3bm_prev=$cpu3bs_prev
cpu3wm_prev=$cpu3ws_prev

# 5-ти Минутные данные
cpub5m_prev=$cpubs_prev
cpuw5m_prev=$cpuws_prev
cpu0b5m_prev=$cpu0bs_prev
cpu0w5m_prev=$cpu0ws_prev
cpu1b5m_prev=$cpu1bs_prev
cpu1w5m_prev=$cpu1ws_prev
cpu2b5m_prev=$cpu2bs_prev
cpu2w5m_prev=$cpu2ws_prev
cpu3b5m_prev=$cpu3bs_prev
cpu3w5m_prev=$cpu3ws_prev

# 15-ти Минутные данные
cpub15m_prev=$cpubs_prev
cpuw15m_prev=$cpuws_prev
cpu0b15m_prev=$cpu0bs_prev
cpu0w15m_prev=$cpu0ws_prev
cpu1b15m_prev=$cpu1bs_prev
cpu1w15m_prev=$cpu1ws_prev
cpu2b15m_prev=$cpu2bs_prev
cpu2w15m_prev=$cpu2ws_prev
cpu3b15m_prev=$cpu3bs_prev
cpu3w15m_prev=$cpu3ws_prev

# Крутим бесконечный цикл
while true; do
sleep 1

#########################################
# Секундные данные                      #
#########################################

# Текущая температура
temp0=$(cat $tempzone0)
temp1=$(cat $tempzone1)

# Используемое пространство HDD в kB (необходимо правильно указать имя своего устройства)
hddused=$(df | grep sdb2 | awk '{print $3}')

# Используемая память в байтах
memused=$(free -b | grep Mem | awk '{print $3}')

# Используемый своп в байтах
swapused=$(free -b | grep Swap | awk '{print $3}')

# Текущая частота процессора
cpufreq=$(cat $cpufreqcur)
#scalingfreq=$(cat $scalingfreqcur)

# Принятые / переданные байты
rx0=$(cat $rx)
tx0=$(cat $tx)

# Читаем текущее значение загрузки процессора
# Загрузку процессора и ядер считаем в % умноженных на 10!!!
cpu=$(cat $loadcpu | grep cpu)

# Время работы системы в секундах
uptime=$(cat /proc/uptime | awk '{print $1}')

dt=$(date +%s)

# Считаем принятые / переданные байты
rxs=$(($rx0-$rxs_prev))
txs=$(($tx0-$txs_prev))
# Сохраняем текущие значения
rxs_prev=$rx0
txs_prev=$tx0

# Считаем среднее значение загрузки процессора
busy=$(echo $cpu | awk '{print ($2+$3+$4)}')
work=$(echo $cpu | awk '{print ($2+$3+$4+$5)}')
cpubs=$(($busy-$cpubs_prev))
cpuws=$(($work-$cpuws_prev))
cpul=$(($cpubs*1000/$cpuws))
# Сохраняем текущие значения
cpubs_prev=$busy
cpuws_prev=$work

# Считаем среднее значение загрузки 0 ядра
busy0=$(echo $cpu | awk '{print ($13+$14+$15)}')
work0=$(echo $cpu | awk '{print ($13+$14+$15+$16)}')
cpu0bs=$(($busy0-$cpu0bs_prev))
cpu0ws=$(($work0-$cpu0ws_prev))
cpu0l=$(($cpu0bs*1000/$cpu0ws))
# Сохраняем текущие значения
cpu0bs_prev=$busy0
cpu0ws_prev=$work0

# Считаем среднее значение загрузки 1 ядра
busy1=$(echo $cpu | awk '{print ($24+$25+$26)}')
work1=$(echo $cpu | awk '{print ($24+$25+$26+$27)}')
cpu1bs=$(($busy1-$cpu1bs_prev))
cpu1ws=$(($work1-$cpu1ws_prev))
cpu1l=$(($cpu1bs*1000/$cpu1ws))
# Сохраняем текущие значения
cpu1bs_prev=$busy1
cpu1ws_prev=$work1

# Считаем среднее значение загрузки 2 ядра
busy2=$(echo $cpu | awk '{print ($35+$36+$37)}')
work2=$(echo $cpu | awk '{print ($35+$36+$37+$38)}')
cpu2bs=$(($busy2-$cpu2bs_prev))
cpu2ws=$(($work2-$cpu2ws_prev))
cpu2l=$(($cpu2bs*1000/$cpu2ws))
# Сохраняем текущие значения
cpu2bs_prev=$busy2
cpu2ws_prev=$work2

# Считаем среднее значение загрузки 3 ядра
busy3=$(echo $cpu | awk '{print ($46+$47+$48)}')
work3=$(echo $cpu | awk '{print ($46+$47+$48+$49)}')
cpu3bs=$(($busy3-$cpu3bs_prev))
cpu3ws=$(($work3-$cpu3ws_prev))
cpu3l=$(($cpu3bs*1000/$cpu3ws))
# Сохраняем текущие значения
cpu3bs_prev=$busy3
cpu3ws_prev=$work3

echo "$dt $cpul $cpu0l $cpu1l $cpu2l $cpu3l $cpufreq $temp0 $temp1 $memused $swapused $hddused $rxs $txs $uptime" >> $outsec
tail -n $hs $outsec > $temp
cp $temp $outsec
# echo "$dt $cpul $cpu0l $cpu1l $cpu2l $cpu3l $cpufreq $temp0 $temp1 $memused $swapused $hddused $rxs $txs $uptime"

#########################################
# Минутные данные                       #
#########################################

if [[ $dt -ge $(($tm+60)) ]]
then

tm=$dt

# Считаем принятые / переданные байты
rxm=$(($rx0-$rxm_prev))
txm=$(($tx0-$txm_prev))
# Сохраняем текущие значения
rxm_prev=$rx0
txm_prev=$tx0

# Считаем среднее значение загрузки процессора
cpubm=$(($busy-$cpubm_prev))
cpuwm=$(($work-$cpuwm_prev))
cpul=$(($cpubm*1000/$cpuwm))
# Сохраняем текущие значения
cpubm_prev=$busy
cpuwm_prev=$work

# Считаем среднее значение загрузки 0 ядра
cpu0bm=$(($busy0-$cpu0bm_prev))
cpu0wm=$(($work0-$cpu0wm_prev))
cpu0l=$(($cpu0bm*1000/$cpu0wm))
# Сохраняем текущие значения
cpu0bm_prev=$busy0
cpu0wm_prev=$work0

# Считаем среднее значение загрузки 1 ядра
cpu1bm=$(($busy1-$cpu1bm_prev))
cpu1wm=$(($work1-$cpu1wm_prev))
cpu1l=$(($cpu1bm*1000/$cpu1wm))
# Сохраняем текущие значения
cpu1bm_prev=$busy1
cpu1wm_prev=$work1

# Считаем среднее значение загрузки 2 ядра
cpu2bm=$(($busy2-$cpu2bm_prev))
cpu2wm=$(($work2-$cpu2wm_prev))
cpu2l=$(($cpu2bm*1000/$cpu2wm))
# Сохраняем текущие значения
cpu2bm_prev=$busy2
cpu2wm_prev=$work2

# Считаем среднее значение загрузки 3 ядра
cpu3bm=$(($busy3-$cpu3bm_prev))
cpu3wm=$(($work3-$cpu3wm_prev))
cpu3l=$(($cpu3bm*1000/$cpu3wm))
# Сохраняем текущие значения
cpu3bm_prev=$busy3
cpu3wm_prev=$work3

echo "$dt $cpul $cpu0l $cpu1l $cpu2l $cpu3l $cpufreq $temp0 $temp1 $memused $swapused $hddused $rxm $txm $uptime" >> $outmin
tail -n $hm $outmin > $temp
cp $temp $outmin

fi

#########################################
# 5 минутные данные                     #
#########################################

if [[ $dt -ge $(($t5m+300)) ]]
then

t5m=$dt

# Считаем принятые / переданные байты
rx5m=$(($rx0-$rx5m_prev))
tx5m=$(($tx0-$tx5m_prev))
# Сохраняем текущие значения
rx5m_prev=$rx0
tx5m_prev=$tx0

# Считаем среднее значение загрузки процессора
cpub5m=$(($busy-$cpub5m_prev))
cpuw5m=$(($work-$cpuw5m_prev))
cpul=$(($cpub5m*1000/$cpuw5m))
# Сохраняем текущие значения
cpub5m_prev=$busy
cpuw5m_prev=$work

# Считаем среднее значение загрузки 0 ядра
cpu0b5m=$(($busy0-$cpu0b5m_prev))
cpu0w5m=$(($work0-$cpu0w5m_prev))
cpu0l=$(($cpu0b5m*1000/$cpu0w5m))
# Сохраняем текущие значения
cpu0b5m_prev=$busy0
cpu0w5m_prev=$work0

# Считаем среднее значение загрузки 1 ядра
cpu1b5m=$(($busy1-$cpu1b5m_prev))
cpu1w5m=$(($work1-$cpu1w5m_prev))
cpu1l=$(($cpu1b5m*1000/$cpu1w5m))
# Сохраняем текущие значения
cpu1b5m_prev=$busy1
cpu1w5m_prev=$work1

# Считаем среднее значение загрузки 2 ядра
cpu2b5m=$(($busy2-$cpu2b5m_prev))
cpu2w5m=$(($work2-$cpu2w5m_prev))
cpu2l=$(($cpu2b5m*1000/$cpu2w5m))
# Сохраняем текущие значения
cpu2b5m_prev=$busy2
cpu2w5m_prev=$work2

# Считаем среднее значение загрузки 3 ядра
cpu3b5m=$(($busy3-$cpu3b5m_prev))
cpu3w5m=$(($work3-$cpu3w5m_prev))
cpu3l=$(($cpu3b5m*1000/$cpu3w5m))
# Сохраняем текущие значения
cpu3b5m_prev=$busy3
cpu3w5m_prev=$work3

echo "$dt $cpul $cpu0l $cpu1l $cpu2l $cpu3l $cpufreq $temp0 $temp1 $memused $swapused $hddused $rx5m $tx5m $uptime" >> $out5min
tail -n $h5m $out5min > $temp
cp $temp $out5min

fi

#########################################
# 15 минутные данные                    #
#########################################

if [[ $dt -ge $(($t15m+900)) ]]
then

t15m=$dt

# Считаем принятые / переданные байты
rx15m=$(($rx0-$rx15m_prev))
tx15m=$(($tx0-$tx15m_prev))
# Сохраняем текущие значения
rx15m_prev=$rx0
tx15m_prev=$tx0

# Считаем среднее значение загрузки процессора
cpub15m=$(($busy-$cpub15m_prev))
cpuw15m=$(($work-$cpuw15m_prev))
cpul=$(($cpub15m*1000/$cpuw15m))
# Сохраняем текущие значения
cpub15m_prev=$busy
cpuw15m_prev=$work

# Считаем среднее значение загрузки 0 ядра
cpu0b15m=$(($busy0-$cpu0b15m_prev))
cpu0w15m=$(($work0-$cpu0w15m_prev))
cpu0l=$(($cpu0b15m*1000/$cpu0w15m))
# Сохраняем текущие значения
cpu0b15m_prev=$busy0
cpu0w15m_prev=$work0

# Считаем среднее значение загрузки 1 ядра
cpu1b15m=$(($busy1-$cpu1b15m_prev))
cpu1w15m=$(($work1-$cpu1w15m_prev))
cpu1l=$(($cpu1b15m*1000/$cpu1w15m))
# Сохраняем текущие значения
cpu1b15m_prev=$busy1
cpu1w15m_prev=$work1

# Считаем среднее значение загрузки 2 ядра
cpu2b15m=$(($busy2-$cpu2b15m_prev))
cpu2w15m=$(($work2-$cpu2w15m_prev))
cpu2l=$(($cpu2b15m*1000/$cpu2w15m))
# Сохраняем текущие значения
cpu2b15m_prev=$busy2
cpu2w15m_prev=$work2

# Считаем среднее значение загрузки 3 ядра
cpu3b15m=$(($busy3-$cpu3b15m_prev))
cpu3w15m=$(($work3-$cpu3w15m_prev))
cpu3l=$(($cpu3b15m*1000/$cpu3w15m))
# Сохраняем текущие значения
cpu3b15m_prev=$busy3
cpu3w15m_prev=$work3

echo "$dt $cpul $cpu0l $cpu1l $cpu2l $cpu3l $cpufreq $temp0 $temp1 $memused $swapused $hddused $rx15m $tx15m $uptime" >> $out15min
tail -n $h15m $out15min > $temp
cp $temp $out15min

fi

done

exit 0